#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, os
import pickle
import string
import time

import traceback

#try:
#  # Python2
#  import Tkinter as tk
#except ImportError:
#  # Python3
#  import tkinter as tk

from   xml.dom.minidom import parse
import xml.dom.minidom

from mod_ddt_request import *
from mod_ddt_data    import *
from mod_utils       import *
from mod_elm         import AllowedList

os.chdir(os.path.dirname(os.path.realpath(sys.argv[0])))

import mod_globals
if mod_globals.os != 'android':
  import mod_ddt_screen

eculist = None 
ecudump = {} # {'request':'response'}

class DDTECU():

  #elm     = None    #elm class
  screen  = None    #screen class
  cecu    = None    #chosen ecu
  ecufname = '' 
  requests = {}
  datas    = {}
  req4data = {}
  langmap  = {}
  defaultEndian = 'Big'
  sentRequests  = []
  BaudRate      = '500000'
  Multipoint    = '1'

  def __init__(self, cecu):
  
    global eculist
  
    self.elm      = 0
    self.cecu     = cecu
    self.ecufname = '' 
    self.requests = {}
    self.datas    = {}
    self.req4data = {}
    self.langmap  = {}
    self.BaudRate   = '500000'
    self.Multipoint = '1'
    
    
  def setELM(self, elm):
    self.elm = elm  
    
  def setLangMap(self,langmap):
    self.langmap  = langmap
    
  def translate(self, data):
    
    #get data instance
    if data in self.datas.keys():
      d = self.datas[data]
    else:
      return data    

    # find appropriate request r
    if data in self.req4data.keys() and self.req4data[data] in self.requests.keys():
      r = self.requests[self.req4data[data]]
    else: return data
    
    sentBytes = r.SentBytes
    startByte = r.ReceivedDI[data].FirstByte
    startBit  = r.ReceivedDI[data].BitOffset
    bitLength = d.BitsCount
    
    if bitLength%8:
      startBit = 7-startBit
      if r.ReceivedDI[data].Endian=="Big":
        startBit = 7 - startBit
    else:
      startBit = 0
    
    key = "%s:%s:%s:%s"%(sentBytes,str(startByte), str(startBit), str(bitLength))
    
    if key in self.langmap.keys():
      return self.langmap[key]
    else:
      return data
    
  def minDistSearch(self, h, el, strong = True):
    ''' try to find nearest ECU if there is no exact math '''
    
    r   = []  #result list
    md  = 0xFFFF #minimum distance
    h1 = h[:-8]  #Soft, Version
    h2 = h[-8:]  #DiagVersion, Supplier
    for k in el.keys():
      k1 = k[:-8]  #Soft, Version
      k2 = k[-8:]  #DiagVersion, Supplier
      
      #print 'h1',h1,'k1',k1,'h2',h2,'k2',k2, el[k]
      
      if len(h1)+3==len(k1):
        h1t = h1[:-3]+"".join("{:02x}".format(ord(c)) for c in h1[-3:])
      else:
        h1t = h1
      
      d = 0
      l = min(len(h1t),len(k1))
      for i in range(2,l):  # minus address
        if k1[i]!='?':
          d = d + abs(ord(h1t[i])-ord(k1[i]))
      
      #Strong rule for VDIAG and Supplier
      if strong and (d!=0 or (len(h1)!=len(k1))): 
        continue
      
      l = min(len(h2),len(k2))
      for i in range(0,l):
        if k2[i]!='?':
          d = d + abs(ord(h2[i])-ord(k2[i]))
      #print '%-50s %s %d'%(el[k],k,d)
      if d<md:
        r = []
        md = d
      if d==md:
        r.append('%-50s %s'%(el[k],k))
    return r
    
  def scanECU( self ):  
    
    global eculist
    
    #local variables
    Address     = ''
    DiagVersion = ''
    Supplier    = ''
    Soft        = ''
    Version     = ''
    hash        = ''

    #try to get response on 2180 command
    print "Getting ID"
    self.clearELMcache()
    IdRsp = self.elm.request( req = '2180', positive = '61', cache = False )
    print 'IdRsp:',IdRsp
    
    '''      0         1         2         3         4         5         6         7      '''
    '''      01234567890123456789012345678901234567890123456789012345678901234567890123456'''
    #IdRsp = '61 80 82 00 44 66 27 44 32 31 33 82 00 38 71 38 00 A7 74 00 56 05 02 01 00 00'
    #IdRsp = '61 80 32 35 35 39 52 05 30 39 36 35 38 36 36 52 00 26 07 01 10 10 02 01 00 88 A3'
    #IdRsp = '61 80 39 39 31 38 52 04 41 41 39 31 34 37 31 52 00 02 00 00 00 00 02 01 00 88 00'
    '''                           -- --------                ----- -----                  '''
    '''              DiagVersion--+      |                     |     +--Version           '''
    '''                        Supplier--+                     +--Soft                    '''
    
    if len(IdRsp)>59:
      Address     = self.cecu['dst']
      DiagVersion = str(int(IdRsp[21:23],16))
      Supplier    = IdRsp[24:32].replace(' ','').decode('hex')
      Soft        = IdRsp[48:53].replace(' ','')
      Version     = IdRsp[54:59].replace(' ','')
      if len(DiagVersion)==1 : DiagVersion = '0'+DiagVersion
      hash        = Address+DiagVersion+Supplier+Soft+Version
    
    print 'Address:"%s" DiagVersion:"%s" Supplier:"%s" Soft:"%s" Version:"%s"'%( Address, DiagVersion, Supplier, Soft, Version)
            
    #make or load eculist 
    print "Loading eculist"
    eculistcache = "./cache/ddt_eculist.p"
    
    if os.path.isfile(eculistcache):                              #if cache exists
      eculist = pickle.load( open( eculistcache, "rb" ) )         #load it
    else:                                                         #else
      self.loadECUlist()                                          #loading original data
      if eculist == None: return                                  #return if no eculist file
      pickle.dump( eculist, open( eculistcache, "wb" ) )          #and save cache
    
    if hash in eculist.keys():
      self.ecufname = '../ecus/'+eculist[hash]
      print "ECU file name: ", self.ecufname
    else:
      print "No such ECU in data base."
      print "Try minimum distance search (Strong)"
      problist = self.minDistSearch(hash,eculist,True)
      if len(problist)==0:
        print "Try minimum distance search (Weak). Be very careful!!!"
        problist = self.minDistSearch(hash,eculist,False)
      
      if len(problist)>10:
        print "Too many variants"
        return
      
      for r in problist:
        print pyren_encode( r )
        
      if len(mod_globals.opt_ddtxml)==0:
        print "You may enter the file name by yourself or left empty to exit"
        fname = raw_input("File name:")
      else:
        fname = mod_globals.opt_ddtxml
        
      fname = fname.strip()
      if len(fname):
        self.ecufname = '../ecus/'+fname
      else:
        print 'Empty file name'
        return
        
    self.loadXml()  
        
  def loadXml( self, xmlfile = '' ):
  
    if len(xmlfile):
      self.ecufname = xmlfile
      
    #Load XML
    xdom = xml.dom.minidom.parse(self.ecufname)
    xdoc = xdom.documentElement
    if not xdoc:
      print "No such file:", self.ecufname
      return
    
    cans = xdoc.getElementsByTagName("CAN")
    if cans:
      for can in cans:
        self.BaudRate = can.getAttribute("BaudRate")
        self.Multipoint = can.getAttribute("Multipoint")  
    
    print "Loading requests"
    rq_class = decu_requests(self.requests, xdoc)
    print "Loading datas"
    dt_class = decu_datas(self.datas, xdoc)
    
    for r in self.requests.values():
      for di in r.ReceivedDI.values():
        if di.Ref or di.Name not in self.req4data.keys():
          self.req4data[di.Name] = r.Name
    
  def loadECUlist(self):
  
    global eculist
    
    #open xml
    self.xmlName = '../ecus/eculist.xml'
    xdom = xml.dom.minidom.parse(self.xmlName)
    xdoc = xdom.documentElement
    if not xdoc:
      print "No file:", self.xmlName
      return None

    eculist = {}
    functions = xdoc.getElementsByTagName("Function")
    if functions:
      for function in functions:
        Address = hex(int(function.getAttribute("Address"))).replace("0x","").zfill(2).upper()
        targets = function.getElementsByTagName("Target")
        if targets:
          for target in targets:
            href = target.getAttribute("href")
            ais = target.getElementsByTagName("AutoIdents")
            if ais:
              for ai in ais:
                AutoIdents = ai.getElementsByTagName("AutoIdent")
                if AutoIdents:
                  for AutoIdent in AutoIdents:
                    DiagVersion = AutoIdent.getAttribute("DiagVersion")
                    Supplier    = AutoIdent.getAttribute("Supplier")
                    Soft        = AutoIdent.getAttribute("Soft")
                    Version     = AutoIdent.getAttribute("Version")
                    if len(DiagVersion)==1 : DiagVersion = '0'+DiagVersion
                    hash = Address+DiagVersion+Supplier+Soft+Version
                    eculist[hash] = href
  
  def saveDump( self ):
    ''' save responces from all 21xx, 22xxxx commands '''
    
    xmlname = self.ecufname
    if xmlname.upper().endswith('.XML'):
      xmlname = xmlname[:-4]
    if xmlname.upper().startswith('../ECUS/'):
      xmlname = xmlname[8:]
    
    dumpname = './dumps/'+str(int(time.time()))+'_'+xmlname+'.txt'    
    df = open(dumpname,'wt')
    
    self.elm.clear_cache()
    
    for request in self.requests.values():
      if request.SentBytes[:2] in AllowedList + ['17','19']:
        pos = chr(ord(request.SentBytes[0])+4)+request.SentBytes[1]
        rsp = self.elm.request(request.SentBytes, pos, False)
        if ':' in rsp: continue
        df.write('%s:%s\n'%(request.SentBytes,rsp))
    
    df.close()
    
  def loadDump( self, dumpname='' ):
    ''' load saved dump for demo mode '''
    
    global ecudump
    
    ecudump = {}
    
    xmlname = self.ecufname
    if xmlname.upper().endswith('.XML'):
      xmlname = xmlname[:-4]
    if xmlname.upper().startswith('../ECUS/'):
      xmlname = xmlname[8:]
    
    if len(dumpname)==0:
      flist = []
      for root, dirs, files in os.walk("./dumps"):
        for f in files:
          if (xmlname+'.') in f:
            flist.append(f)
            
      if len(flist)==0: return
      flist.sort()
      dumpname = os.path.join("./dumps/", flist[-1])
      
    df = open(dumpname,'rt')
    lines = df.readlines()
    df.close()
    
    for l in lines:
      l = l.strip().replace('\n','')
      if l.count(':')==1:
        req,rsp = l.split(':')
        ecudump[req] = rsp
    
    self.elm.setDump( ecudump )
    
  def clearELMcache(self):
    self.elm.clear_cache()
    self.sentRequests = []

  def elmRequest(self, req, delay='0', positive='', cache=True):
    ''' dispath requestst to elm '''
    if req.startswith('10'):
      self.elm.startSession = req
      
    #strange definition of delays in ddt database
    if delay>0 and delay<1000: delay = 1000
    
    rsp = self.elm.request(req, positive, cache , serviceDelay=delay)
    
    #log this request to ddt log
    if self.screen != None and (not cache or req not in self.sentRequests):
      self.screen.addToLog('>'+req+'  Rcvd:'+rsp)
    
    #cache it
    if cache and req not in self.sentRequests:
      self.sentRequests.append( req )

    return rsp

  def getValue(self, data, auto=True, request=None, responce=None):
    ''' extract and format value '''
    
    # first get hex value
    hv = self.getHex( data, auto, request, responce )
    
    # get data instance
    if data in self.datas.keys():
      d = self.datas[data]
    else:
      return 'NoDatasItem'    
    
    # list
    if len(d.List.keys()):
      listIndex = int(hv,16)
      if listIndex in d.List.keys():
        hv = hex(listIndex)[2:]
        return hv+':'+d.List[listIndex]
      else:
        return hv
    
    # scaled   
    if d.Scaled:
      # conver to int
      p = int(hv,16)
      # if it negative signed value
      if d.signed and p>(2**(d.BitsCount-1)-1):
        p = p-2**d.BitsCount
      # calculate the formula
      res = p*float(d.Step)/float(d.DivideBy)+float(d.Offset)
      # format the result
      if len(d.Format) and '.' in d.Format:
        acc = len(d.Format.split('.')[1])
        fmt = '%.'+str(acc)+'f'
        res = fmt%(res)
      res = str(res)
      # remove '.0' from the end 
      if res.endswith('.0'): res = res[:-2]
      #add units and return
      return res+' '+d.Unit
    
    # just bytes 
    if d.BytesASCII:
      res = hv.decode('hex')
      
      if not all(c in string.printable for c in res): 
        res = hv

      #debug
      #print '>>>>>>>>>>',hv
      #print '#'*50
      #for line in traceback.format_stack():
      #  print(line.strip())

      return res 

    return hv

  def getHex(self, data, auto=True, request=None, responce=None):
    ''' extract hex value from response '''
    
    # d will be a data instace
    if data in self.datas.keys():
      d = self.datas[data]
    else:
      return '00'    

    # find appropriate request r
    if request==None:
      if data in self.req4data.keys() and self.req4data[data] in self.requests.keys():
        r = self.requests[self.req4data[data]]
      else: return '00'
    else:
      r = request
    
    #check if command only for manual send or require parameters
    if auto and (r.ManuelSend or len(r.SentDI.keys())>0):
      return '00'
      
    #protect not expert mode
    if (r.SentBytes[:2] not in AllowedList) and not mod_globals.opt_exp: 
      return '00'

    #if response not defined as an argument
    if responce==None:
      # send new request or get response from cache
      resp = self.elmRequest( r.SentBytes )
    else:
      resp = responce
    
    #format and check the response
    resp = resp.strip().replace(' ','')
    if not all(c in string.hexdigits for c in resp): resp = ''
    resp = ' '.join(a+b for a,b in zip(resp[::2], resp[1::2]))
      
    #prepare parameters for extraction
    littleEndian = True if r.ReceivedDI[data].Endian=="Little" else False
    sb    = r.ReceivedDI[data].FirstByte - 1 
    bits  = d.BitsCount
    sbit  = r.ReceivedDI[data].BitOffset
    bytes = (bits+sbit-1)/8 + 1
    if littleEndian:
      rshift = sbit
    else:
      rshift = ((bytes+1)*8 - (bits+sbit))%8
    
    #check length of responce
    if (sb*3+bytes*3-1)>(len(resp)):
      return '00'
    
    #extract hex
    hexval = resp[sb*3:(sb+bytes)*3-1]
    hexval = hexval.replace(" ","")

    #shift and mask
    val = (int(hexval,16)>>int(rshift))&(2**bits-1)

      
    #format result
    hexval = hex(val)[2:]
    #remove 'L'
    if hexval[-1:].upper()=='L':
      hexval = hexval[:-1]
    #add left zero if need 
    if len(hexval)%2:
      hexval = '0'+hexval

    #revert byte order if little endian
    if littleEndian:
      a = hexval
      b = ''
      if not len(a) % 2:
        for i in range(0,len(a),2):
          b = a[i:i+2]+b
        hexval = b
    
    return hexval

  def packValues( self, requestName, iValues ): 
    ''' pack values from iValues to command                            '''
    ''' return string                                                  '''
    ''' if cathe the error then return string begining with ERROR: word'''
    ''' else return command in hex                                     '''
    
    # get request instance
    r = self.requests[requestName]
    
    # get command pattern
    cmdPatt = r.SentBytes
    
    # for every DataItem
    for sdi in r.SentDI.values():
    
      #get DataItem instance 
      d = self.datas[sdi.Name]
      if d.Name not in iValues.keys():
        print 'WARNING: not defined value:%s' % d.Name
        continue
        #return 'ERROR: not defined value:%s' % d.Name
      
      #get value  
      value = iValues[d.Name].get().strip()
      
      # list
      if len(d.List.keys()) and ':' in value:
        value = value.split(':')[0]
        
      # scaled
      if d.Scaled:
        #if there is units then remove them
        if ' ' in value:
          value = value.split(' ')[0]
        #check 0x
        if value.upper().startswith('0X'):
          value = value[2:]
        else:  #calculate reverse formula
          if not all(c in string.digits for c in value):
            return 'ERROR: Wrong value in parametr:%s (it should have %d bytes), be decimal or starts with 0x for hex' % (d.Name, d.BytesCount)
          flv = (float( value ) - float(d.Offset))*float(d.DivideBy)/float(d.Step)
          value = hex(int(flv))
        
      # ascii 
      if d.BytesASCII:
        hst = ''
        for c in value:
          hst = hst + hex(ord(c))[2:].zfill(2)
        value = hst

      #prepare parameters for extraction
      littleEndian = True if sdi.Endian=="Little" else False
      sb    = sdi.FirstByte - 1 
      bits  = d.BitsCount
      sbit  = sdi.BitOffset
      bytes = (bits+sbit-1)/8 + 1
      if littleEndian:
        lshift = sbit
      else:
        lshift = ((bytes+1)*8 - (bits+sbit))%8
        
      # shift value on bit offset 
      val = int(value,16)
      val = (val&(2**bits-1))<<lshift
      value = hex(val)[2:]
      #remove 'L'
      if value[-1:].upper()=='L':
        value = value[:-1]
      #add left zero if need 
      if len(value)%2:
        value = '0'+value

      # check hex
      if value.upper().startswith('0X'): value = value[2:]
      value = value.zfill(bytes*2).upper()
      if not all(c in string.hexdigits for c in value) and len(value)==bytes*2:
        return 'ERROR: Wrong value in parametr:%s (it should have %d bytes)' % (d.Name, d.BytesCount)

      #prepare base and mask
      base = cmdPatt[sb*2:(sb+bytes)*2]
      binbase  = int(base,16)
      binvalue = int(value,16)
      mask     = (2**bits-1)<<lshift
      
      #shift and mask
      binvalue = binbase ^ (mask & binbase) | binvalue

      #remove '0x'
      value = hex(binvalue)[2:].upper()
      #remove 'L'
      if value[-1:].upper()=='L':
        value = value[:-1]
      value = value[-bytes*2:].zfill(bytes*2)

      cmdPatt = cmdPatt[0:sb*2] + value + cmdPatt[(sb+bytes)*2:]

    return cmdPatt

