#!/usr/bin/python
# -*- coding: utf-8 -*-


import sys, os
import operator

import xml.dom.minidom

os.chdir(os.path.dirname(os.path.realpath(sys.argv[0])))

import mod_globals
from   mod_utils    import *

class DataItem:
  Name      = ""
  Endian    = ""
  FirstByte = 1
  BitOffset = 0
  Ref       = False
  
  def __str__(self):
    out = 'Endian = %-5s FirstByte = %2d BitOffset = %2d  Ref = %1d Name = %s' % (self.Endian, 
       self.FirstByte, self.BitOffset, self.Ref, self.Name)
    return pyren_encode(out)  

  def __init__(self, di, defaultEndian ):
    self.Name  = di.getAttribute("Name")
    Endian = di.getAttribute("Endian")
    if Endian: 
      self.Endian=Endian
    else:
      self.Endian=defaultEndian
    FirstByte = di.getAttribute("FirstByte")
    if FirstByte: self.FirstByte=int(FirstByte)
    BitOffset = di.getAttribute("BitOffset")
    if BitOffset: self.BitOffset=int(BitOffset)
    Ref = di.getAttribute("Ref")
    if Ref:
      if Ref=='1': self.Ref=True
      else:        self.Ref=False
  
class decu_request:

  Name                  = ""
  DossierMaintenabilite = None
  ManuelSend            = None
  DenyAccess            = []
  SentBytes             = ""
  SentDI                = {}
  VariableLength        = False
  ReceivedDI            = {}
  MinBytes              = 1
  ShiftBytesCount       = 0
  ReplyBytes            = ""
  
  def __str__(self):
  
    sd = ''
    for s in sorted(self.SentDI.values(), key=operator.attrgetter("FirstByte","BitOffset")):
      sd = sd + '\n'+str(s)
    sd = pyren_decode(sd)
    
    rd = ''
    for r in sorted(self.ReceivedDI.values(), key=operator.attrgetter("FirstByte","BitOffset")): 
      rd = rd + '\n'+str(r)
    rd = pyren_decode(rd)
    
    out = '''
  Name                  = %s
  DossierMaintenabilite = %d
  ManuelSend            = %d
  DenyAccess            = %s
  SentBytes             = %s
  SentDI                = %s
  VariableLength        = %d
  ReceivedDI            = %s
  MinBytes              = %d
  ShiftBytesCount       = %d
  ReplyBytes            = %s
    ''' % (self.Name, self.DossierMaintenabilite, self.ManuelSend, self.DenyAccess, self.SentBytes, 
           sd, self.VariableLength, rd, self.MinBytes, self.ShiftBytesCount, self.ReplyBytes)
    return pyren_encode(out)  
  
  def __init__(self, rq, defaultEndian ):
    self.Name  = rq.getAttribute("Name")

    self.DossierMaintenabilite = False
    DossierMaintenabilite = rq.getElementsByTagName("DossierMaintenabilite").item(0)
    if DossierMaintenabilite: self.DossierMaintenabilite=True

    self.ManuelSend = False
    ManuelSend = rq.getElementsByTagName("ManuelSend")
    if ManuelSend: self.ManuelSend=True

    self.DenyAccess = []
    DenyAccess = rq.getElementsByTagName("DenyAccess").item(0)
    if DenyAccess:
      NoSDS = DenyAccess.getElementsByTagName("NoSDS")
      if NoSDS: self.DenyAccess.append('NoSDS')

      Supplier = DenyAccess.getElementsByTagName("Supplier")
      if Supplier: self.DenyAccess.append('Supplier')

      Engineering = DenyAccess.getElementsByTagName("Engineering")
      if Engineering: self.DenyAccess.append('Engineering')

      Plant = DenyAccess.getElementsByTagName("Plant")
      if Plant: self.DenyAccess.append('Plant')

      AfterSales = DenyAccess.getElementsByTagName("AfterSales")
      if AfterSales: self.DenyAccess.append('AfterSales')
    
    
    Sent = rq.getElementsByTagName("Sent").item(0)
    if Sent:
      self.SentBytes = ""
      SentBytes = Sent.getElementsByTagName("SentBytes")
      if SentBytes: self.SentBytes = SentBytes.item(0).firstChild.nodeValue

      self.VariableLength = False
      VariableLength = Sent.getElementsByTagName("VariableLength")
      if VariableLength: self.VariableLength=True

      self.SentDI = {}
      DataItems = Sent.getElementsByTagName("DataItem")
      if DataItems:
        for di in DataItems:
          dataitem = DataItem( di, defaultEndian )
          self.SentDI[dataitem.Name]=dataitem
          
    Received = rq.getElementsByTagName("Received").item(0)
    if Received:
      self.MinBytes = 0
      MinBytes = Received.getAttribute("MinBytes")
      if MinBytes: self.MinBytes = int(MinBytes)

      self.ShiftBytesCount = 0
      ShiftBytesCount = Received.getElementsByTagName("ShiftBytesCount")
      if ShiftBytesCount: self.ShiftBytesCount = int(ShiftBytesCount.item(0).firstChild.nodeValue)

      self.ReplyBytes = ""
      ReplyBytes = Received.getElementsByTagName("ReplyBytes")
      if ReplyBytes: self.ReplyBytes = ReplyBytes.item(0).firstChild.nodeValue

      self.ReceivedDI = {}
      DataItems = Received.getElementsByTagName("DataItem")
      if DataItems:
        for di in DataItems:
          dataitem = DataItem( di, defaultEndian  )
          self.ReceivedDI[dataitem.Name]=dataitem

class decu_requests:
  def __init__(self, requiest_list, xdoc):
    #try to find default endian
    tmpdoc = xdoc.getElementsByTagName("Requests")
    defaultEndian = ''
    if tmpdoc:
      defaultEndian = tmpdoc[0].getAttribute("Endian")
    requests = xdoc.getElementsByTagName("Request")
    if requests:
      for rq in requests:
        request = decu_request( rq, defaultEndian )
        requiest_list[request.Name] = request
        #print request
        
