#!/usr/bin/python
# -*- coding: utf-8 -*-
 
import pprint
import gc
import os
#import traceback


from   mod_utils    import *

try:
  # Python2
  import Tkinter as tk
  import ttk
  import tkFont
  import tkMessageBox
  import tkFileDialog
  import tkSimpleDialog
except ImportError:
  # Python3
  import tkinter as tk
  import tkinter.ttk as ttk
  import tkFont
  import tkMessageBox        
  import tkFileDialog

from   xml.dom.minidom import parse
import xml.dom.minidom

import mod_globals
from mod_elm	import AllowedList

import string
import copy
import time

class screenSettings(): # for future use. 
  geometry = ''   #main window geometry
  scf      = 1.0  #font scale coefficient 
        
class ButtonConfirmationDialog(tkSimpleDialog.Dialog):
  def __init__(self, parent, text ):
    self.top = tk.Toplevel(parent)
    self.top.title('Please confirm')
    self.top.geometry("450x450+50+50")
    self.top.bind('<Escape>', self.no)
    self.confirm = False
    
    txt = tk.Text(self.top, relief=tk.GROOVE, borderwidth=1, wrap="none" ) #, width=190, heigth=180 
    txt.insert(tk.END, text+'\n')
    txt.pack(side=tk.TOP,padx=5,pady=5,expand=True, fill='both')
    
    frame = tk.Frame( self.top )
    frame.pack(side=tk.BOTTOM)
    
    byes = tk.Button(frame, text="YES", command=self.yes)
    byes.pack(side=tk.LEFT,padx=5,pady=5)
    
    bno = tk.Button(frame, text="NO", command=self.no)
    bno.pack(side=tk.RIGHT,padx=5,pady=5)
    
  def no(self):
    self.top.destroy()
    self.confirm = False

  def yes(self):
    self.top.destroy()
    self.confirm = True
    
  def show(self):
    self.top.deiconify()
    self.top.wait_window()
    return self.confirm

    
class InfoDialog(tkSimpleDialog.Dialog):
  def __init__(self, parent, text):
    self.top = tk.Toplevel(parent)
    self.top.title('Info')
    self.top.geometry("800x500+50+50")
    self.top.bind('<Escape>', self.close)
    
    txt = tk.Text(self.top, relief=tk.GROOVE, borderwidth=1, wrap="none")
    txt.insert(tk.END, text+'\n')
    txt.pack(side=tk.TOP,padx=5,pady=5,expand=True, fill='both')
    #txt.config(state=tk.DISABLED)
    
    b = tk.Button(self.top, text="Close", command=self.close)
    b.pack(side=tk.BOTTOM,padx=5,pady=5)
    
    #self.initial_focus.focus_set()
    
  def close(self):
    self.top.destroy()

class DDTScreen(tk.Frame):

  tl           = 0
  updatePeriod = 200
  decu     = None
  xmlName  = ''
  Screens  = {}
  dDisplay = {}
  dValue   = {}  #display value indexed by DataName
  iValue   = {}  #input   value indexed by DataName
  dReq     = {}  #request names
  sReq_dl  = {}  #delays for requests sent thru starting the screen
  sReq_lst = []  #list of requests sent thru starting the screen (order is important)
  dBtnSend = {}
  dFrames  = []
  dObj     = []  #objects for place_forget
  tObj     = {}  #objects for text captions
  start    = True
  
  jid      = None  #for after_cancel
  jdsu     = None  
  
  firstResize = False
  update_dInputs = True
  
  currentscreen = None
  
  scf      = 1.0   #font scale factor
  
  def __init__(self, ddtFileName, xdoc, decu):
    
    self.xmlName = ddtFileName
    self.xdoc    = xdoc

    #init local variables
    self.Screens = {}
    #self.Labels  = {}
    self.decu    = decu
    self.decu.screen  = self
    
    # init window
    self.root = tk.Tk() 
    self.root.option_add('*Dialog.msg.font', 'Courier New 12')
    #self.root.overrideredirect(True)  
    self.root.geometry("1024x768")
    tk.Frame.__init__(self, self.root)
    self.root.bind('<plus>', self.fontUp)
    self.root.bind('<=>',    self.fontUp)
    self.root.bind('<minus>', self.fontDown)
       
    #self.root.wm_attributes('-fullscreen', 1)
    
    self.translated = tk.BooleanVar()
    self.translated.set(True)
    
    self.expertmode = tk.BooleanVar()
    self.expertmode.set(mod_globals.opt_exp)
    
    self.approve = tk.BooleanVar()
    self.approve.set(True)
    
    self.initUI()
    self.tl = time.time()
    self.updateScreen()
    self.root.focus_force()
    self.root.mainloop()  

  def __del__(self):
    if self.jid is not None:
      self.root.after_cancel(self.jid)
    if self.jdsu is not None:
      self.root.after_cancel(self.jdsu)
    self.decu.screen  = None
    #self.root.destroy()
    
  def updateScreen(self):
    '''Send periodic requests''' 
    
    #clear elm cache
    self.decu.clearELMcache()
    
    #execute all req with no ManuelSend
    for r in self.dReq.keys():
      req = self.decu.requests[r].SentBytes
      
      #not expert mode protection
      if (req[:2] not in AllowedList) and not mod_globals.opt_exp: 
        self.addToLog('Req:'+req+' rejected due to non expert mode')
        continue
      
      if not self.dReq[r]:
        rsp = self.decu.elmRequest(req)
        #self.addToLog('Sent:'+req+'  Rcvd:'+rsp)
    
    #update all variables from dDisplay
    for d in self.dValue.keys():
      val = self.decu.getValue(d)
      if ':'in val:
        self.dValue[d].set( val.split(':')[1] )
      else: 
        self.dValue[d].set( val ) 
    
      #update variable in dInputs
      if self.update_dInputs:
        if d in self.iValue.keys():
          if len(self.decu.datas[d].List) or self.decu.datas[d].BytesASCII or self.decu.datas[d].Scaled:
            self.iValue[d].set( val )
          else:
            val = self.decu.getHex(d)
            self.iValue[d].set( '0x'+val )

    self.update_dInputs = False

    # re-launch update in x milliseconds
    tb = time.time()
    self.jid = self.root.after(int(self.updatePeriod-(tb - self.tl)*100), self.updateScreen)
    self.tl = tb  
    
  def startScreen(self): 
    '''Send requests during starting of the screen'''

    #clear elm cache
    self.decu.clearELMcache()
    
    #execute all req with no ManuelSend
    self.addToLog('#Starting screen')
    for r in self.sReq_lst:
      req = self.decu.requests[r].SentBytes
      
      #not expert mode protection
      if (req[:2] not in ['10']+AllowedList) and not mod_globals.opt_exp: 
        self.addToLog('Request:'+req+' rejected due to non expert mode')
        continue
      
      rsp = self.decu.elmRequest(req, self.sReq_dl[r])
      #self.addToLog('(Start)Sent:'+req+'  Rcvd:'+rsp)
      
    self.addToLog('#Finish starting screen')
    
    #update all variables from dDisplay
    for d in self.dValue.keys():
      val = self.decu.getValue(d)      
      if ':'in val:
        val = val.split(':')[1]
      self.dValue[d].set( val ) 
    
    #update all variables from dInputs
    for d in self.iValue.keys():
      val = self.decu.getValue(d)
      if len(self.decu.datas[d].List) or self.decu.datas[d].BytesASCII or self.decu.datas[d].Scaled:
        self.iValue[d].set( val )
      else:
        val = self.decu.getHex(d)
        self.iValue[d].set( '0x'+val )
        
  def buttonPressed(self, btn, key):
    self.addToLog( 'Button pressed:'+btn )
    
    #stop periodic requests
    if self.jid is not None:
      self.root.after_cancel(self.jid)

    #clear elm cache
    self.decu.clearELMcache()
            
    #prepare list of commands
    if key in self.dBtnSend.keys():
      sends = self.dBtnSend[key]
    else: return    

    #pack DataItems to command
    slist = [] # list of smap
    smap  = {} # {'d':delay, 'c':sendCmd, 'r':requestInstance}
    error = False
    
    for send in sends:
      delay       = send['Delay']
      requestName = send['RequestName']
      r = self.decu.requests[requestName]
      
      sendCmd = self.decu.packValues( requestName, self.iValue )
      if 'ERROR' in sendCmd:
        self.addToLog(sendCmd)
        error = True
      smap['d'] = delay
      smap['c'] = sendCmd
      smap['r'] = r
      slist.append(copy.deepcopy(smap))
     
    #show confirmation dialog if approve is True
    confirmed = True     
    if not error and self.approve.get():
      commandSet = '\n\n'
      for c in slist:
        commandSet += "%-10s Delay:%-3s (%s)\n"%(c['c'],c['d'],c['r'].Name)
      confirmed = ButtonConfirmationDialog(self.root, commandSet).show()

    #send commands
    if not error and confirmed:
      self.addToLog('#Starting command execution')
      for c in slist: 
        rsp = '00'
        
        #protection for debug period
        if mod_globals.opt_exp:
          rsp = self.decu.elmRequest(c['c'],c['d'],cache=False)
        else:
          rsp = r.ReplyBytes.replace(' ','')
          rsp = ' '.join(a+b for a,b in zip(rsp[::2], rsp[1::2]))
          self.addToLog("#  Should be sent:%s Delay:%s" % (c['c'], c['d']))
        
        r = c['r']
        if len(r.ReceivedDI.keys()):
          #update relevant values
          for d in r.ReceivedDI.keys():
            val = self.decu.getValue(d, False, r, rsp) #allow execute commands with manuelSend
            if d in self.dValue.keys():
              if ':' in val:
                self.dValue[d].set( val.split(':')[1] )
              else:
                self.dValue[d].set( val )            
            if d in self.iValue.keys():
              if len(self.decu.datas[d].List) or self.decu.datas[d].BytesASCII or self.decu.datas[d].Scaled:
                self.iValue[d].set( val )
              else:
                val = self.decu.getHex(d, False, r, rsp)
                self.iValue[d].set( '0x'+val )
              
        self.addToLog('#Finish command execution')
        if not mod_globals.opt_exp:
          self.addToLog("#WARNING!!! NOT EXPERT MODE!!! Commands wasn't realy sent to ECU")
     
    #re-launch periodic requests
    if self.start:
      self.jid = self.root.after(self.updatePeriod, self.updateScreen)
      
    #request to update dInputs
    self.update_dInputs = True

  
  def OnScreenChange(self, item):
    self.loadScreen(self.Screens[item])
            
  def resizeEvent( self, event ):
    #filter first resize
    if self.firstResize:
      self.firstResize = False
      return
    #stop previous resize update query
    if self.jdsu is not None:
      self.root.after_cancel(self.jdsu)
    #make new resize update query
    if self.currentscreen:
      self.jdsu = self.root.after(500, lambda: self.loadScreen( self.currentscreen ))
           
  def saveDump(self):
    self.decu.saveDump()
    
  def loadDump(self):
    fname = tkFileDialog.askopenfilename(defaultextension=".txt",
                         filetypes=[("Dump files",".txt")],
                         initialdir="dumps",
                         title="Choose Dump")
    self.decu.loadDump( fname )

  def repaint(self):
    self.loadScreen(self.currentscreen)
  
  def changeMode(self):
    mod_globals.opt_exp = self.expertmode.get()
    if mod_globals.opt_exp: 
      self.modeLabel.configure(text="Expert Mode", background='#d9d9d9', foreground='#d90000')
    else:
      self.modeLabel.configure(text="ReadOnly Mode", background='#d9d9d9', foreground='#000000')      
      
  def fontUp(self, event=None):
    self.scf = self.scf * 1.25
    self.loadScreen(self.currentscreen)

  def fontDown(self, event=None):
    self.scf = self.scf * 0.8
    self.loadScreen(self.currentscreen)
    
  def exit(self):
    if self.jid is not None:
      self.root.after_cancel(self.jid)
    self.root.destroy()

  def startStop(self):
    if self.start:
      #stop it
      self.start = False
      self.sessionmenu.entryconfig( 0, label='Start' )
      self.startStopButton.configure( text='Start' )
      if self.jid is not None:
        self.root.after_cancel(self.jid)
    else:
      #start it
      self.start = True
      self.sessionmenu.entryconfig( 0, label='Stop' )
      self.startStopButton.configure( text='Stop' )
      self.jid = self.root.after(self.updatePeriod, self.updateScreen)
    return
         
  def addToLog(self, log):
    self.eculog.insert(tk.END, log+'\n')
    self.eculog.see(tk.END)
     
  def OnTreeClick(self, event):
    iid = self.tree.focus()
    if iid in self.Screens.keys():
      #self.addToLog(iid)
      self.loadScreen(self.Screens[iid])
      
  def ddtColor(self, s):
    si = int(s)
    if si<0: si = si * -1 + 0x800000
    c = hex(si).replace("0x","").zfill(6).upper()
    return '#'+c[4:6]+c[2:4]+c[0:2]
    
  def clearLogs(self):
    self.eculog.delete(1.0, tk.END)
  
  def saveLogs(self):
    fname = ''
    fname = tkFileDialog.asksaveasfilename(defaultextension=".txt",
                         filetypes=[("Text files",".txt")],
                         initialdir="logs",
                         title="Save as")
    if len(fname):
      with open(fname, "w") as log:
        log.write(self.eculog.get(1.0, tk.END))
      
  def readDTC(self):
    tkMessageBox.showinfo("Read DTC", "Under construction. Use pyren screens.")
     
  def clearDTC(self):
    tkMessageBox.showinfo("Clear DTC", "Under construction. Use pyren screens.")

  def calcPIN(self):
    tkMessageBox.showinfo("PIN calculator", "Under construction")

  def rightButtonClicked(self, event, tag=''):
    id = self.ddt.find_closest(event.x, event.y)[0]

    if id not in self.tObj.keys() and tag=='': return
    
    if tag!='':
      closest = tag
    else:
      closest = self.tObj[self.ddt.find_closest(event.x, event.y)[0]]
      
    d = str(self.decu.datas[closest])
    r = ''
    if closest in self.decu.req4data.keys() and \
             self.decu.req4data[closest] in self.decu.requests.keys():
      r = str(self.decu.requests[self.decu.req4data[closest]])
    
    xText = d+'\n'+'*'*50 +'\n'+r
    dialog = InfoDialog(self.root, xText)
    self.root.wait_window(dialog.top)
    
  def initUI(self):
  
    if mod_globals.os == 'nt':
      self.scf = 1.0   #scale font
    else:
      self.scf = 1.25  #scale font
    
    self.root.title(self.xmlName)
    self.style = ttk.Style()
    self.style.theme_use('classic')
    
    self.screen_width  = self.root.winfo_screenwidth()
    self.screen_height = self.root.winfo_screenheight()
    
    bot_h =  50 if self.screen_height>850 else 20
    lef_w = 200 if self.screen_width>1000 else 0
    
    #print self.screen_width, self.screen_height

    #create main layout    
    self.tbpw = tk.PanedWindow(self.root, orient=tk.VERTICAL,background='#d9d9d9'
                              ,handlepad=0,sashpad=0,sashrelief=tk.GROOVE,showhandle=1)
    #top farme (Main Screen)
    self.top_f = tk.Frame(self.tbpw,background='#d9d9d9')

    #bottom farme (Ecu Log)
    self.bot_f = tk.Frame(self.tbpw,background='#d9d9d9')
    self.eculog = tk.Text(self.bot_f, relief=tk.GROOVE, borderwidth=1, height=4 )
    self.eculog.pack(fill=tk.BOTH, expand=True)

    self.tbpw.add(self.top_f, height = 748)
    self.tbpw.add(self.bot_f, height = bot_h )
    
    self.tbpw.pack(fill=tk.BOTH, expand=True)
    self.root.update()
    
    #create menu
    self.menubar = tk.Menu(self.root)

    self.sessionmenu = tk.Menu(self.menubar, tearoff=0)
    self.sessionmenu.add_command(label="Stop", command=self.startStop)
    self.sessionmenu.add_command(label="Exit", command=self.exit)
    self.menubar.add_cascade(label="Session", menu=self.sessionmenu)

    self.viewmenu = tk.Menu(self.menubar, tearoff=0)
    self.viewmenu.add_command(label="Font Increse  (+)", command=self.fontUp)
    self.viewmenu.add_command(label="Font Decrease (-)", command=self.fontDown)
    self.menubar.add_cascade(label="View", menu=self.viewmenu)

    self.logsmenu = tk.Menu(self.menubar, tearoff=0)
    self.logsmenu.add_command(label="Clear logs", command=self.clearLogs, accelerator="Ctrl+k")
    self.logsmenu.add_command(label="Save to file", command=self.saveLogs, accelerator="Ctrl+s")
    self.menubar.add_cascade(label="Logs", menu=self.logsmenu)

    self.DTCmenu = tk.Menu(self.menubar, tearoff=0)
    self.DTCmenu.add_command(label="Read DTC", command=self.readDTC)
    self.DTCmenu.add_command(label="Clear DTC", command=self.clearDTC)
    self.menubar.add_cascade(label="DTC", menu=self.DTCmenu)

    self.toolmenu = tk.Menu(self.menubar, tearoff=0)
    self.toolmenu.add_command(label="Calculate PIN", command=self.calcPIN)
    self.toolmenu.add_separator()
    if mod_globals.opt_demo:
      self.toolmenu.add_command(label="Load DUMP", command=self.loadDump)
    else:
      self.toolmenu.add_command(label="Save DUMP", command=self.saveDump)
    self.menubar.add_cascade(label="Tools", menu=self.toolmenu)

    self.settingsmenu = tk.Menu(self.menubar, tearoff=0)
    self.settingsmenu.add_separator()
    self.settingsmenu.add_checkbutton(label="Tranlate", onvalue=True, offvalue=False, variable=self.translated, command=self.repaint)
    self.settingsmenu.add_separator()
    self.settingsmenu.add_checkbutton(label="Approve commands", onvalue=True, offvalue=False, variable=self.approve)
    self.settingsmenu.add_separator()
    self.settingsmenu.add_separator()
    self.settingsmenu.add_checkbutton(label="Expert Mode ( be careful !!! )", onvalue=True, offvalue=False, variable=self.expertmode, command=self.changeMode)    
    self.menubar.add_cascade(label="Settings", menu=self.settingsmenu)

    self.ecumenu = tk.Menu(self.menubar, tearoff=0)
    categs = self.xdoc.getElementsByTagName("Category")
    if categs:
      for cat in categs:
        catname = cat.getAttribute("Name")
        catmenu = tk.Menu(self.ecumenu, tearoff=0)
        self.ecumenu.add_cascade(label=catname, menu=catmenu)
        screens = cat.getElementsByTagName("Screen")
        if screens:
          for scr in screens:
            scrname = scr.getAttribute("Name")
            self.Screens[scrname] = scr
            catmenu.add_command(label=scrname, command=lambda item=scrname: self.OnScreenChange(item))
    self.menubar.add_cascade(label="Screens", menu=self.ecumenu)
    self.root.config(menu=self.menubar)


    self.lrpw = tk.PanedWindow(self.top_f, orient=tk.HORIZONTAL,background='#d9d9d9'
                               ,handlepad=0,sashpad=0,sashrelief=tk.GROOVE,showhandle=1)
    #left farme
    self.lef_f = tk.Frame(self.lrpw,background='#d9d9d9')
    #right farme
    self.rig_f = tk.Frame(self.lrpw,background='#d9d9d9')
    self.lrpw.add(self.lef_f, width = lef_w)
    self.lrpw.add(self.rig_f, width = 800)
    self.lrpw.pack(fill=tk.BOTH, expand=True)
    self.root.update()
    
    if mod_globals.opt_exp: 
      self.modeLabel = tk.Label(self.lef_f, height = 1, text="Expert Mode", background='#d9d9d9', foreground='#d90000')
    else:
      self.modeLabel = tk.Label(self.lef_f, height = 1, text="ReadOnly Mode", background='#d9d9d9', foreground='#000000')      
    self.modeLabel.pack(side=tk.TOP)   
    
    btnFrame = tk.Frame(self.lef_f,background='#d9d9d9')
    btnFrame.pack(side=tk.TOP, fill=tk.X)
    self.exitButton      = ttk.Button(btnFrame, text="Exit", command=self.exit)
    self.exitButton.pack(side=tk.RIGHT, expand=True)
    self.startStopButton = ttk.Button(btnFrame, text="Stop", command=self.startStop)
    self.startStopButton.pack(side=tk.LEFT, expand=True)
    
    #add treeView on left frame
    self.tree = ttk.Treeview(self.lef_f)
    self.tree.heading('#0', text='Screens', anchor='w')
    categs = self.xdoc.getElementsByTagName("Category")
    if categs:
      for cat in categs:
        catname = cat.getAttribute("Name")
        self.tree.insert("", "end", catname, text=catname, open=True)
        screens = cat.getElementsByTagName("Screen")
        if screens:
          for scr in screens:
            scrname = scr.getAttribute("Name")
            iid = self.tree.insert(catname, "end", text=scrname)
            self.Screens[iid] = scr
    self.tree.bind("<<TreeviewSelect>>", self.OnTreeClick)
    self.tree.pack(side=tk.BOTTOM, fill=tk.BOTH, expand=True)
    
    ######################################################################################

    #self.ddt = tk.Canvas(self.rig_f, relief=tk.GROOVE, width = 600, height = 500)

    ######################################################################################
    
    # Mad way to do scrollable convas
    
    self.ddtcnv = tk.Canvas(self.rig_f, relief=tk.GROOVE, width = 600, height = 600)
    self.vbar =tk.Scrollbar(self.rig_f,orient=tk.VERTICAL, command=self.ddtcnv.yview)
    self.ddtcnv.config(yscrollcommand=self.vbar.set)
    
    self.vbar.pack(side=tk.RIGHT,fill=tk.Y)
    self.ddtcnv.pack(side=tk.LEFT,expand=True,fill=tk.BOTH)
    
    self.ddtcnv.xview_moveto(0)
    self.ddtcnv.yview_moveto(0)
    
    self.ddtfrm = tk.Frame(self.ddtcnv, width = 600, height = 600)
    self.ddtfrm_id = self.ddtcnv.create_window((0,0), window=self.ddtfrm, anchor=tk.NW)
    #self.ddtfrm.pack(expand=True,fill=tk.BOTH)

    self.ddt = tk.Canvas(self.ddtfrm, relief=tk.GROOVE, width = 600, height = 600)
    self.ddt.pack(expand=True,fill=tk.BOTH)
    
    ######################################################################################

    if os.name=='posix':
      self.ddt.bind("<Button-2>", self.rightButtonClicked)
    else:
      self.ddt.bind("<Button-3>", self.rightButtonClicked)

    #self.ddt.pack(fill=tk.BOTH, expand=True)
    self.ddtfrm.bind("<Configure>", self.confFrm)
    self.ddtcnv.bind("<Configure>", self.confDdt)
    
    self.rig_f.bind("<Configure>", self.resizeEvent)
    
  def confFrm(self, event):
    size = (self.ddtfrm.winfo_reqwidth(), self.ddtfrm.winfo_reqheight())
    self.ddtcnv.config(scrollregion="0 0 %s %s" % size)
    if self.ddtfrm.winfo_reqwidth() != self.ddtcnv.winfo_width():
      self.ddtcnv.config(width=self.ddtfrm.winfo_reqwidth())
        
  def confDdt(self, event):
    if self.ddtfrm.winfo_reqwidth() != self.ddtcnv.winfo_width():
      self.ddtcnv.itemconfigure(self.ddtfrm_id, width=self.ddtcnv.winfo_width())
      
  def minInputHeight( self, scr ):
    _minInputHeight = 0xffff
    inputs = scr.getElementsByTagName("Input")
    if inputs:
      for input in inputs:
        xRect = input.getElementsByTagName("Rectangle").item(0)
        if xRect:
          xrHeight  = int(xRect.getAttribute("Height"))
          if xrHeight<_minInputHeight:
            _minInputHeight = xrHeight
    return _minInputHeight

  def minButtonHeight( self, scr ):
    _minButtonHeight = 0xffff
    inputs = scr.getElementsByTagName("Button")
    if inputs:
      for input in inputs:
        xRect = input.getElementsByTagName("Rectangle").item(0)
        if xRect:
          xrHeight  = int(xRect.getAttribute("Height"))
          if xrHeight<_minButtonHeight:
            _minButtonHeight = xrHeight
    return _minButtonHeight
    
  def loadScreen(self, scr):
  
    #debug
    #print '#'*50
    #for line in traceback.format_stack():
    #    print(line.strip())

    self.currentscreen = scr
    
    self.firstResize = True

    scr_w = int(scr.getAttribute("Width"))
    scr_h = int(scr.getAttribute("Height"))
    bg_color = scr.getAttribute("Color")
    
    scx = 1  #scale X
    scy = 1  #scale Y
    
    #access scale
    max_x = 0.0
    max_y = 0.0
    recs = scr.getElementsByTagName("Rectangle")
    for rec in recs:
      xrLeft    = int(rec.getAttribute("Left"))
      xrTop     = int(rec.getAttribute("Top"))
      xrHeight  = int(rec.getAttribute("Height"))
      xrWidth   = int(rec.getAttribute("Width"))
      w = xrLeft+xrWidth
      h = xrTop+xrHeight
      if w > max_x and w < scr_w*3:
        max_x = w
      if h > max_y and h < scr_h*3:
        max_y = h
      #print xrLeft, xrTop, xrHeight, xrWidth
    
    #main frame re-create
    self.ddt.delete('all')
    self.ddt.update_idletasks()

    for o in self.dObj:
      o.place_forget()
    for f in self.dFrames:
      f.place_forget()
      
    self.dValue   = {}  #value indexed by DataName
    self.iValue   = {}  #value indexed by DataName param with choise
    self.dReq     = {}  #request names
    self.sReq_dl  = {}  #delays for requests sent thru starting the screen
    self.sReq_lst = []  #list of requests sent thru starting the screen (order is important)
    self.dDisplay = {}  #displays object for renew
    self.dObj     = []  #objects for place_forget  
    self.tObj     = {}  #objects for text captions
    self.dFrames  = []  #container frames
    gc.collect() 
    
    scx = int(max_x / self.ddtcnv.winfo_width() + 1)
    scy = int(max_y / self.ddtcnv.winfo_height() + 1)
    
    if scy<10: scy=10
    if scx<10: scx=10
    
    #print '*'*50
    #print 'max:',max_x,max_y, ' winfo:', self.ddtcnv.winfo_width(), self.ddtcnv.winfo_height()

    _minInputHeight = int(self.minInputHeight( scr )) / 25
    _minButtonHeight = int(self.minButtonHeight( scr )) / 25

    scx = min([scx,20])
    scy = min([scy,_minInputHeight,_minButtonHeight,20])
    
    #print [scy,_minInputHeight,_minButtonHeight], scx
    
    #set scroll region
    self.ddtcnv.config(scrollregion=(0, 0, max_x/scx, max_y/scy))
    #self.ddtfrm.config(width=max_x/scx, height=max_y/scy)
    self.ddt.config(width=max_x/scx, height=max_y/scy, bg=self.ddtColor(bg_color))
    self.ddtcnv.xview_moveto(0)
    self.ddtcnv.yview_moveto(0)

    if os.name=='posix':
      os_event = "<Button-2>"
    else:
      os_event = "<Button-3>"
    
    #load labels (just descriptions of fields)
    labels = scr.getElementsByTagName("Label")
    if labels:
      for label in labels:
        xText      = label.getAttribute("Text")
        xColor     = label.getAttribute("Color")
        xAlignment = label.getAttribute("Alignment")

        xRect = label.getElementsByTagName("Rectangle").item(0)
        if xRect:
          xrLeft    = int(xRect.getAttribute("Left"))   /scx
          xrTop     = int(xRect.getAttribute("Top"))    /scy
          xrHeight  = int(xRect.getAttribute("Height")) /scy
          xrWidth   = int(xRect.getAttribute("Width"))  /scx

        xFont = label.getElementsByTagName("Font").item(0)
        if xFont:
          xfName    = xFont.getAttribute("Name")
          xfSize    = xFont.getAttribute("Size")
          xfBold    = xFont.getAttribute("Bold")
          xfItalic  = xFont.getAttribute("Italic")
          xfColor   = xFont.getAttribute("Color")

        self.ddt.create_rectangle(xrLeft, xrTop, xrLeft+xrWidth, xrTop+xrHeight, fill=self.ddtColor(xColor), outline=self.ddtColor(xColor) )
        
        if xText=='New label': continue
        if xColor==xfColor: continue
 
        xfSize = str(int(float(xfSize)*self.scf))

        if xrLeft<0: xrLeft=0
        if xrTop <0: xrTop =0

        xfBold='bold'     if xfBold  =='1' else 'normal'
        xfItalic='italic' if xfItalic=='1' else 'roman'
        
        #if xAlignment=='0': continue
        
        if xAlignment=='1':
          xrTop=xrTop+xrHeight/2
          xAlignment='w'
        elif xAlignment=='2':
          xrLeft = xrLeft+xrWidth/2
          xAlignment='n'
        else:
          xAlignment='nw'

        lFont = tkFont.Font(family=xfName, size=xfSize, weight=xfBold)
        id = self.ddt.create_text(xrLeft, xrTop, text=xText, font=lFont, width=xrWidth, anchor=xAlignment, fill=self.ddtColor(xfColor))

    #load displays (show values)
    dispalys = scr.getElementsByTagName("Display")
    if dispalys:
      for dispaly in dispalys:
        xAlignment = '0'
        xText      = dispaly.getAttribute("DataName")
        xReq       = dispaly.getAttribute("RequestName")
        self.dReq[xReq] = self.decu.requests[xReq].ManuelSend
        xColor     = dispaly.getAttribute("Color")
        xWidth     = int(dispaly.getAttribute("Width")) /scx

        xRect = dispaly.getElementsByTagName("Rectangle").item(0)
        if xRect:
          xrLeft    = int(xRect.getAttribute("Left"))   /scx
          xrTop     = int(xRect.getAttribute("Top"))    /scy
          xrHeight  = int(xRect.getAttribute("Height")) /scy
          xrWidth   = int(xRect.getAttribute("Width"))  /scx

        xFont = dispaly.getElementsByTagName("Font").item(0)
        if xFont:
          xfName    = xFont.getAttribute("Name")
          xfSize    = xFont.getAttribute("Size")
          xfBold    = xFont.getAttribute("Bold")
          xfItalic  = xFont.getAttribute("Italic")
          xfColor   = xFont.getAttribute("Color")

        self.dDisplay[xText] = 1

        self.ddt.create_rectangle(xrLeft, xrTop, xrLeft+xrWidth, xrTop+xrHeight, fill=self.ddtColor(xColor), outline=self.ddtColor(xColor) )
        
        if xColor==xfColor: continue
 
        xfSize = str(int(float(xfSize)*self.scf))

        if xrLeft<0: xrLeft=0
        if xrTop <0: xrTop =0

        xfBold='bold'     if xfBold  =='1' else 'normal'
        xfItalic='italic' if xfItalic=='1' else 'roman'

        if xAlignment=='1':
          xAlignment='w'
        elif xAlignment=='2':
          xrLeft = xrLeft+xrWidth/2
          xAlignment='center'
        else:
          xAlignment='w'
                  
        lFont = tkFont.Font(family=xfName, size=xfSize, weight=xfBold)
        
        if xWidth>40:
          if self.translated.get():
            tText = self.decu.translate( xText )
          else:
            tText = xText
          id = self.ddt.create_text(xrLeft, xrTop+xrHeight/2, text=tText, font=lFont, width=xrWidth, anchor=xAlignment, fill=self.ddtColor(xfColor))
          self.tObj[id] = xText

        frame = tk.Frame(self.ddt, width=xrWidth-xWidth, height=xrHeight, relief=tk.GROOVE, borderwidth=0)

        frame.place(x=xrLeft+xWidth, y=xrTop)
        self.dFrames.append( frame )
        
        if xText not in self.dValue.keys():
          self.dValue[xText] = tk.StringVar()
          self.dValue[xText].set('0')
          
        obj = tk.Label(frame, text=self.dValue[xText], relief=tk.GROOVE, borderwidth=1, font=lFont, textvariable=self.dValue[xText])

        obj.bind(os_event, lambda event, tag=xText: self.rightButtonClicked( event, tag ))

        obj.place(width=xrWidth-xWidth, height=xrHeight)
        self.dObj.append( obj )

    #load Inputs (permits enter or choose values)
    inputs = scr.getElementsByTagName("Input")
    if inputs:
      for input in inputs:
        xAlignment = '0'
        xText      = input.getAttribute("DataName")
        xReq       = input.getAttribute("RequestName")
        xColor     = input.getAttribute("Color")
        xWidth     = int(input.getAttribute("Width")) /scx

        xRect = input.getElementsByTagName("Rectangle").item(0)
        if xRect:
          xrLeft    = int(xRect.getAttribute("Left"))   /scx
          xrTop     = int(xRect.getAttribute("Top"))    /scy
          xrHeight  = int(xRect.getAttribute("Height")) /scy
          xrWidth   = int(xRect.getAttribute("Width"))  /scx

        xFont = input.getElementsByTagName("Font").item(0)
        if xFont:
          xfName    = xFont.getAttribute("Name")
          xfSize    = xFont.getAttribute("Size")
          xfBold    = xFont.getAttribute("Bold")
          xfItalic  = xFont.getAttribute("Italic")
          xfColor   = xFont.getAttribute("Color")

        self.ddt.create_rectangle(xrLeft, xrTop, xrLeft+xrWidth, xrTop+xrHeight, fill=self.ddtColor(xColor), outline=self.ddtColor(xColor) )
        
        if xColor==xfColor: continue
 
        xfSize = str(int(float(xfSize)*self.scf))

        if xrLeft<0: xrLeft=0
        if xrTop <0: xrTop =0

        xfBold='bold'     if xfBold  =='1' else 'normal'
        xfItalic='italic' if xfItalic=='1' else 'roman'
          
        if xAlignment=='1':
          xrTop=xrTop+xrHeight/2
          xAlignment='w'
        elif xAlignment=='2':
          xrTop=xrTop+xrHeight/2
          xrLeft = xrLeft+xrWidth/2
          xAlignment='center'
        else:
          xAlignment='w'

        lFont = tkFont.Font(family=xfName, size=xfSize, weight=xfBold)
        if xWidth>40:
          if self.translated.get():
            tText = self.decu.translate( xText )
          else:
            tText = xText
          id = self.ddt.create_text(xrLeft, xrTop+xrHeight/2, text=tText, font=lFont, width=xrWidth, anchor=xAlignment, fill=self.ddtColor(xfColor))
          self.tObj[id] = xText
        
        frame = tk.Frame(self.ddt, width=xrWidth-xWidth, height=xrHeight, relief=tk.GROOVE, borderwidth=0)

        frame.place(x=xrLeft+xWidth, y=xrTop)
        self.dFrames.append( frame )

        if xText not in self.iValue.keys():
          self.iValue[xText] = tk.StringVar()

        if xText in self.decu.datas.keys() and len(self.decu.datas[xText].List.keys()):
          optionList = []
          if xText not in self.iValue.keys():
            self.iValue[xText] = tk.StringVar()
          for i in self.decu.datas[xText].List.keys():
            optionList.append(hex(int(i)).replace("0x","").upper()+':'+self.decu.datas[xText].List[i])
          self.iValue[xText].set(optionList[0])
          obj = tk.OptionMenu(frame, self.iValue[xText], *optionList)
        else:
          self.iValue[xText].set('Enter heare')
          obj = tk.Entry(frame, relief=tk.GROOVE, borderwidth=1, font=lFont, textvariable=self.iValue[xText])
        
        obj.bind(os_event, lambda event, tag=xText: self.rightButtonClicked( event, tag ))

        obj.place(width=xrWidth-xWidth, height=xrHeight)
        self.dObj.append( obj )

    #load buttons
    buttons = scr.getElementsByTagName("Button")
    if buttons:
      for button in buttons:
        xText      = button.getAttribute("Text")

        xRect = button.getElementsByTagName("Rectangle").item(0)
        if xRect:
          xrLeft    = int(xRect.getAttribute("Left"))   /scx
          xrTop     = int(xRect.getAttribute("Top"))    /scy
          xrHeight  = int(xRect.getAttribute("Height")) /scy
          xrWidth   = int(xRect.getAttribute("Width"))  /scx

        xFont = button.getElementsByTagName("Font").item(0)
        if xFont:
          xfName    = xFont.getAttribute("Name")
          xfSize    = xFont.getAttribute("Size")
          xfBold    = xFont.getAttribute("Bold")
          xfItalic  = xFont.getAttribute("Italic")
          xfColor   = xFont.getAttribute("Color")

        xSends = button.getElementsByTagName("Send")
        slist = []
        if xSends:
          slist = []
          for xSend in xSends:
            smap = {}
            xsDelay       = xSend.getAttribute("Delay")
            xsRequestName = xSend.getAttribute("RequestName")
            smap['Delay']       = xsDelay
            smap['RequestName'] = xsRequestName
            if len(xsRequestName)>0:
              slist.append(smap)
        if len(slist):
          #print str(slist)
          self.dBtnSend[str(slist)] = slist
          #self.dBtnSend[xText] = slist

        xfSize = str(int(float(xfSize)*self.scf))

        if xrLeft<0: xrLeft=0
        if xrTop <0: xrTop =0

        xfBold='bold'     if xfBold  =='1' else 'normal'
        xfItalic='italic' if xfItalic=='1' else 'roman'
          
        lFont = tkFont.Font(family=xfName, size=xfSize, weight=xfBold)

        frame = tk.Frame(self.ddt, width=xrWidth, height=xrHeight)
        frame.place(x=xrLeft, y=xrTop)
        self.dFrames.append( frame )
    
        obj = tk.Button(frame, text=xText, font=lFont, relief=tk.GROOVE, command=lambda key=str(slist),btn=xText: self.buttonPressed(btn,key))
        obj.place(width=xrWidth, height=xrHeight)
        self.dObj.append( obj )
    
    #load sends
    sends = scr.getElementsByTagName("Send")
    if sends:
      for send in sends:
        if send.parentNode.isSameNode(scr):
          sDelay       = send.getAttribute("Delay")
          sRequestName = send.getAttribute("RequestName")
          self.sReq_dl[sRequestName] = sDelay
          self.sReq_lst.append(sRequestName)
    if len(self.sReq_lst):
      self.startScreen()
        
        
            
def main():
  print 'exit'  

if __name__ == '__main__':
  main()  
